@@ -23,6 +23,8 @@ __all__ = [
     'histogram',
     'link',
     'mapping',
+    'megares_data_finder',
+    'megares_zip_parser',
     'mlst_profile',
     'mlst_reporter',
     'pubmlst_getter',
